import os
from lxml import etree

bank_to_customer_account_report = 'camt.052.001.06.xsd'
bank_to_customer_statement = 'camt.053.001.06.xsd'
bank_to_customer_debit_credit_notification = 'camt.054.001.06.xsd'
account_reporting_request = 'camt.060.001.03.xsd'

customer_credit_transfer_initiation = 'pain.001.001.08.xsd'
customer_payment_status_report = 'pain.002.001.08.xsd'
customer_payment_reversal = 'pain.007.001.07.xsd'
customer_direct_debit_initiation = 'pain.008.001.07.xsd'
mandate_initiation_request = 'pain.009.001.05.xsd'
mandate_amendment_request = 'pain.010.001.05.xsd'
mandate_cancellation_request = 'pain.011.001.05.xsd'
mandate_acceptance_report = 'pain.012.001.05.xsd'
mandate_copy_request = 'pain.017.001.01.xsd'
mandate_suspension_request = 'pain.018.001.01.xsd'

def get_schema(schema_name):
    return etree.XMLSchema(etree.parse(os.path.join(os.path.dirname(__file__), 'schemas', schema_name)))

def validate(schema_name, tree):
    return get_schema(schema_name).validate(tree)

def validate_or_error(schema_name, tree):
    return get_schema(schema_name).assertValid(tree)



from lxml import etree
from .messages import\
    bank_to_customer_statement as btcs,\
    customer_direct_debit_initiation as cddi,\
    mandate_initiation_request as mir,\
    mandate_amendment_request as mar,\
    mandate_cancellation_request as mcr

def reverse(structure, name = ''):
    new_structure = {
        '_self': name
    }

    for child in structure:
        if not child.startswith('_'):
            sub = structure[child]

            if isinstance(sub, list):
                if isinstance(sub[0], dict):
                    new_structure[sub[0]['_self']] = [reverse(sub[0], child)]
                else:
                    new_structure[sub[0]] = [child]
            elif isinstance(sub, dict):
                # if '_self' in sub:
                new_structure[sub['_self']] = reverse(sub, child)
                # else:
                    # print('WEIRD WEIRD', child, sub)
            else:
                new_structure[sub] = child

    return new_structure

# Reverse message structures
bank_to_customer_statement = reverse(btcs)
customer_direct_debit_initiation = reverse(cddi)
mandate_initiation_request = reverse(mir)
mandate_amendment_request = reverse(mar)
mandate_cancellation_request = reverse(mcr)

def parse_tree(structure, tag):
    data = {}

    for child in tag:
        substructure = structure[child.tag]

        if isinstance(substructure, list):
            # print(substructure[0])
            if isinstance(substructure[0], dict):
                key = substructure[0]['_self']
                value = parse_tree(substructure[0], child)
            else:
                # print('NOPENOPENOPE', type(substructure[0]))
                key = substructure[0]
                value = child.text

            # print(child.tag, key)
            if not key in data:
                data[key] = []
            data[key].append(value)
        elif isinstance(substructure, dict):
            data[substructure['_self']] = parse(substructure, child)
        else:
            data[substructure] = child.text

    return data

def parse(structure, tree):
    if tree.tag == 'Document':
        return parse_tree(structure, tree.child[0])
    return parse_tree(structure, tree)

def parse_string(structure, tree, **kwargs):
    return parse(structure, etree.fromstring(tree, **kwargs))
